package org.zeroturnaround.javarebel.integration.generic;

import java.net.URL;
import org.zeroturnaround.javarebel.ClassResource;
import org.zeroturnaround.javarebel.LoggerFactory;
import org.zeroturnaround.javarebel.integration.support.BaseClassResourceSource;
import org.zeroturnaround.javarebel.integration.util.ResourceUtil;
import org.zeroturnaround.javarebel.support.FallbackClassResource;
import org.zeroturnaround.javarebel.support.FileClassResource;
import org.zeroturnaround.javarebel.support.JarClassResource;

public class FindResourceClassResourceSource extends BaseClassResourceSource {  
  private ClassLoader classloader;
  
  public FindResourceClassResourceSource(ClassLoader classloader) {
    this.classloader = classloader;
  }
  
  public ClassResource getLocalResource(String name) {
    URL url = ResourceUtil.findResource(classloader, name);
    
    if (url == null)
      return null;
    
    LoggerFactory.getInstance().log("Found URL '" + url + "' for resource '" + name + "'.");
    
    return asResource(url);
  }

  public ClassResource[] getLocalResources(String name) {
    URL[] urls = ResourceUtil.findResources(classloader, name);
    
    if (urls == null)
      return null;
    
    ClassResource[] result = new ClassResource[urls.length];
    for (int i = 0; i < urls.length; i++) {
      result[i] = asResource(urls[i]);
      LoggerFactory.getInstance().log("Found URL '" + urls[i] + "' for resource '" + name + "'.");
    }
    return result;
  }
  
  private ClassResource asResource(URL url) {
    if (url.getProtocol().equals("file") && url.getPath().endsWith(".class")) {
      return new FileClassResource(url);
    }
    else if ((url.getProtocol().equals("jar") || url.getProtocol().equals("zip") || url.getProtocol().equals("code-source")) && (url.getPath().indexOf('!') != -1)) {
      return new JarClassResource(url);
    }
    else {
      return new FallbackClassResource(url);
    }
  }
}